@@ -1,4 +1,4 @@
-# Copyright (c) 2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2017-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -18,12 +18,14 @@ gi.require_version('Secret', '1')
 gi.require_version('GtkSpell', '3.0')
 from gi.repository import Gtk, Gio, GLib, Gdk, WebKit2
 
+from threading import current_thread
 from gettext import gettext as _
 from pickle import dump, load
 from urllib.parse import urlparse
 from getpass import getuser
 from time import time
 import json
+from signal import signal, SIGINT, SIGTERM
 
 from eolie.settings import Settings, SettingsDialog
 from eolie.window import Window
@@ -41,12 +43,13 @@ from eolie.menu_pages import PagesMenu
 from eolie.helper_dbus import DBusHelper
 from eolie.helper_task import TaskHelper
 from eolie.define import EOLIE_DATA_PATH, TimeSpan, TimeSpanValues, LoadingType
-from eolie.utils import is_unity, wanted_loading_type
+from eolie.utils import is_unity, wanted_loading_type, set_proxy_from_gnome
+from eolie.logger import Logger
 
 
 class Application(Gtk.Application):
     """
-        Eolie application:
+        Eolie application
     """
 
     __FAVICONS_PATH = "/tmp/eolie_%s" % getuser()
@@ -59,13 +62,20 @@ class Application(Gtk.Application):
         """
         self.__version = version
         self.__state_cache = []
+        signal(SIGINT, lambda a, b: self.quit())
+        signal(SIGTERM, lambda a, b: self.quit())
+        # Set main thread name
+        # We force it to current python 3.6 name, to be sure in case of
+        # change in python
+        current_thread().setName("MainThread")
+        set_proxy_from_gnome()
         # First check WebKit2 version
         if WebKit2.MINOR_VERSION < 18:
             exit("You need WebKit2GTK >= 2.18")
         Gtk.Application.__init__(
-                            self,
-                            application_id="org.gnome.Eolie",
-                            flags=Gio.ApplicationFlags.HANDLES_COMMAND_LINE)
+            self,
+            application_id="org.gnome.Eolie",
+            flags=Gio.ApplicationFlags.HANDLES_COMMAND_LINE)
         self.set_property("register-session", True)
         # Fix proxy for python
         proxy = GLib.environ_getenv(GLib.get_environ(), "all_proxy")
@@ -86,13 +96,10 @@ class Application(Gtk.Application):
                     break
         self.sync_worker = None  # Not initialised
         self.__extension_dir = extension_dir
-        self.debug = False
         self.show_tls = False
         self.cursors = {}
         GLib.set_application_name('Eolie')
         GLib.set_prgname('org.gnome.Eolie')
-        self.add_main_option("debug", b'd', GLib.OptionFlags.NONE,
-                             GLib.OptionArg.NONE, "Debug Eolie", None)
         self.add_main_option("private", b'p', GLib.OptionFlags.NONE,
                              GLib.OptionArg.NONE, "Add a private page",
                              None)
@@ -103,11 +110,10 @@ class Application(Gtk.Application):
                              GLib.OptionFlags.NONE,
                              GLib.OptionArg.NONE, "Do not use cache for art",
                              None)
-        self.add_main_option("show-tls", b's', GLib.OptionFlags.NONE,
+        self.add_main_option("show-tls", b't', GLib.OptionFlags.NONE,
                              GLib.OptionArg.NONE, "Show TLS info",
                              None)
         self.connect("activate", self.__on_activate)
-        self.connect("shutdown", lambda a: self.__save_state())
         self.connect("handle-local-options", self.__on_handle_local_options)
         self.connect("command-line", self.__on_command_line)
         self.register(None)
@@ -133,9 +139,9 @@ class Application(Gtk.Application):
 
         show_sidebar = self.settings.get_value("show-sidebar")
         sidebar_action = Gio.SimpleAction.new_stateful(
-                                       "sidebar",
-                                       None,
-                                       GLib.Variant.new_boolean(show_sidebar))
+            "sidebar",
+            None,
+            GLib.Variant.new_boolean(show_sidebar))
         sidebar_action.connect("change-state", self.__on_sidebar_change_state)
         self.add_action(sidebar_action)
 
@@ -143,10 +149,6 @@ class Application(Gtk.Application):
         shortcuts_action.connect("activate", self.__on_shortcuts_activate)
         self.add_action(shortcuts_action)
 
-        # help_action = Gio.SimpleAction.new('help', None)
-        # help_action.connect('activate', self.__on_help_activate)
-        # self.add_action(help_action)
-
         quit_action = Gio.SimpleAction.new("quit", None)
         quit_action.connect("activate", lambda x, y: self.quit())
         self.add_action(quit_action)
@@ -158,11 +160,11 @@ class Application(Gtk.Application):
         """
         rules = self.adblock.get_default_css_rules()
         self.__default_style_sheet = WebKit2.UserStyleSheet(
-                                 rules,
-                                 WebKit2.UserContentInjectedFrames.ALL_FRAMES,
-                                 WebKit2.UserStyleLevel.USER,
-                                 None,
-                                 None)
+            rules,
+            WebKit2.UserContentInjectedFrames.ALL_FRAMES,
+            WebKit2.UserStyleLevel.USER,
+            None,
+            None)
 
     def do_startup(self):
         """
@@ -238,13 +240,16 @@ class Application(Gtk.Application):
                                    None)
                 self.__profiles = PROFILES
         except Exception as e:
-            print("Application::set_profiles():", e)
+            Logger.error("Application::set_profiles(): %s", e)
 
     def quit(self, vacuum=False):
         """
             Quit application
             @param vacuum as bool
         """
+        self.__save_state()
+        for window in self.windows:
+            window.hide()
         # Stop pending tasks
         self.download_manager.cancel()
         self.adblock.stop()
@@ -253,13 +258,14 @@ class Application(Gtk.Application):
         if active_id != TimeSpan.FOREVER:
             atime = time()
             if active_id != TimeSpan.NEVER:
-                atime -= TimeSpanValues[active_id]/1000000
+                atime -= TimeSpanValues[active_id] / 1000000
             self.history.clear_to(int(atime))
-        # If sync is running, to avoid DB lock, we do not vacuum
-        if self.sync_worker is not None and self.sync_worker.syncing:
-            self.sync_worker.stop()
-            Gio.Application.quit(self)
-        elif vacuum:
+
+        if self.sync_worker is not None:
+            if self.sync_worker.syncing:
+                self.sync_worker.stop()
+            self.sync_worker.save_pendings()
+        if vacuum:
             task_helper = TaskHelper()
             task_helper.run(self.__vacuum,
                             callback=(lambda x: Gio.Application.quit(self),))
@@ -351,24 +357,17 @@ class Application(Gtk.Application):
         # Add a global DBus helper
         self.helper = DBusHelper()
         # First init sync worker
-        from eolie.mozilla_sync import SyncWorker
+        from eolie.firefox_sync import SyncWorker
         if SyncWorker.check_modules():
             self.sync_worker = SyncWorker()
-            # Run a first sync in 10 seconds, speed up app start
-            GLib.timeout_add_seconds(10,
-                                     self.sync_worker.sync,
-                                     False)
-            # Then run a sync every hour
-            GLib.timeout_add_seconds(3600,
-                                     self.sync_worker.sync,
-                                     True)
+            self.sync_worker.sync_loop()
         else:
             self.sync_worker = None
         if self.prefers_app_menu():
             menu = self.get_app_menu()
             self.set_app_menu(menu)
         cssProviderFile = Gio.File.new_for_uri(
-                'resource:///org/gnome/Eolie/application.css')
+            'resource:///org/gnome/Eolie/application.css')
         cssProvider = Gtk.CssProvider()
         cssProvider.load_from_file(cssProviderFile)
         screen = Gdk.Screen.get_default()
@@ -377,13 +376,12 @@ class Application(Gtk.Application):
                                              Gtk.STYLE_PROVIDER_PRIORITY_USER)
         self.history = DatabaseHistory()
         self.bookmarks = DatabaseBookmarks()
-        # We store cursors for main thread
-        SqlCursor.add(self.history)
-        SqlCursor.add(self.bookmarks)
         self.websettings = DatabaseSettings()
         self.adblock = DatabaseAdblock()
+        self.adblock.create_db()
         self.adblock.update()
         self.phishing = DatabasePhishing()
+        self.phishing.create_db()
         self.adblock_exceptions = DatabaseExceptions("adblock")
         # Do not remove this!
         self.update_default_style_sheet()
@@ -406,7 +404,7 @@ class Application(Gtk.Application):
         # Check MOZ_PLUGIN_PATH
         if self.settings.get_value('enable-plugins') and\
                 not GLib.getenv("MOZ_PLUGIN_PATH"):
-            print("You need to set MOZ_PLUGIN_PATH to use plugins")
+            Logger.info("You need to set MOZ_PLUGIN_PATH to use plugins")
 
         # https://wiki.ubuntu.com/Unity/LauncherAPI
         self.__unity = None
@@ -415,7 +413,7 @@ class Application(Gtk.Application):
                 gi.require_version('Unity', '7.0')
                 from gi.repository import Unity
                 self.__unity = Unity.LauncherEntry.get_for_desktop_id(
-                                                     "org.gnome.Eolie.desktop")
+                    "org.gnome.Eolie.desktop")
             except:
                 pass
 
@@ -428,6 +426,7 @@ class Application(Gtk.Application):
         self.add_action(shortcut_action)
         self.set_accels_for_action("win.shortcut::expose", ["<Alt>e"])
         self.set_accels_for_action("win.exceptions::site", ["<Control>e"])
+        self.set_accels_for_action("win.shortcut::jsblock", ["<Control>j"])
         self.set_accels_for_action("win.shortcut::show_left_panel", ["F9"])
         self.set_accels_for_action("win.shortcut::uri",
                                    ["<Control>l", "<Control>b"])
@@ -455,9 +454,9 @@ class Application(Gtk.Application):
         self.set_accels_for_action("app.settings",
                                    ["<Control>s"])
         self.set_accels_for_action("win.shortcut::backward",
-                                   ["<Alt>Left", "XF86Back"])
+                                   ["<Alt>Left", "Back"])
         self.set_accels_for_action("win.shortcut::forward",
-                                   ["<Alt>Right", "XF86Forward"])
+                                   ["<Alt>Right", "Forward"])
         self.set_accels_for_action("win.shortcut::next",
                                    ["<Control>Tab",
                                     "<Control>Page_Down"])
@@ -476,6 +475,8 @@ class Application(Gtk.Application):
                                    ["<Control>KP_Subtract", "<Control>minus"])
         self.set_accels_for_action("win.shortcut::zoom_default",
                                    ["<Control>KP_0", "<Control>0"])
+        self.set_accels_for_action("win.shortcut::mse_enabled",
+                                   ["<Control>m"])
 
     def __vacuum(self):
         """
@@ -500,7 +501,7 @@ class Application(Gtk.Application):
                 sql.execute("VACUUM")
                 sql.isolation_level = ""
         except Exception as e:
-            print("Application::__vacuum(): ", e)
+            Logger.error("Application::__vacuum(): %s ", e)
         self.art.vacuum()
 
     def __get_state(self, window):
@@ -555,7 +556,7 @@ class Application(Gtk.Application):
             dump(window_states,
                  open(EOLIE_DATA_PATH + "/session_states.bin", "wb"))
         except Exception as e:
-            print("Application::__save_state()", e)
+            Logger.error("Application::__save_state(): %s", e)
 
     def __clean_state_cache(self, window_id):
         """
@@ -588,16 +589,16 @@ class Application(Gtk.Application):
                          ephemeral, state) in window["states"]:
                         loading_type = wanted_loading_type(i)
                         webkit_state = WebKit2.WebViewSessionState(
-                                                         GLib.Bytes.new(state))
+                            GLib.Bytes.new(state))
                         items.append((uri, title, atime, ephemeral,
                                       webkit_state, loading_type))
                         i += 1
                     new_window.container.add_webviews(items)
             elif windows:
-                    size = windows[0]["size"]
-                    maximized = windows[0]["maximized"]
+                size = windows[0]["size"]
+                maximized = windows[0]["maximized"]
         except Exception as e:
-            print("Application::__create_initial_windows()", e)
+            Logger.error("Application::__create_initial_windows(): %s", e)
         if not self.get_windows():
             self.get_new_window(size, maximized)
 
@@ -608,7 +609,7 @@ class Application(Gtk.Application):
             @param options as GLib.VariantDict
         """
         if options.contains("version"):
-            print("Eolie %s" % self.__version)
+            Logger.info("Eolie %s", self.__version)
             return 0
         return -1
 
@@ -621,9 +622,6 @@ class Application(Gtk.Application):
         self.__externals_count = 0
         args = app_cmd_line.get_arguments()
         options = app_cmd_line.get_options_dict()
-        if options.contains("debug"):
-            GLib.setenv("WEBKIT_DEBUG", "network", True)
-            self.debug = True
         if options.contains("show-tls"):
             self.show_tls = True
         if options.contains("disable-artwork-cache"):
@@ -631,7 +629,8 @@ class Application(Gtk.Application):
         ephemeral = options.contains("private")
         if not self.get_windows():
             self.__create_initial_windows(len(args) < 2)
-        if options.contains("new"):
+            active_window = self.active_window
+        elif options.contains("new") or len(args) == 1:
             active_window = self.get_new_window()
         else:
             active_window = self.active_window
@@ -651,16 +650,19 @@ class Application(Gtk.Application):
                 items.append((uri, uri, 0, ephemeral, None, loading_type))
                 i += 1
             active_window.container.add_webviews(items)
-            active_window.present_with_time(Gtk.get_current_event_time())
+            # https://bugzilla.gnome.org/show_bug.cgi?id=766284
+            monotonic_time = int(GLib.get_monotonic_time() / 1000)
+            active_window.present_with_time(monotonic_time)
         # Add default start page
         if not active_window.container.views:
             active_window.container.add_webview(self.start_page,
                                                 LoadingType.FOREGROUND,
                                                 ephemeral)
-        if self.debug:
+        if self.settings.get_value("debug"):
             WebKit2.WebContext.get_default().get_plugins(None,
                                                          self.__on_get_plugins,
                                                          None)
+        Gdk.notify_startup_complete()
         return 0
 
     def __close_window(self, window):
@@ -670,10 +672,9 @@ class Application(Gtk.Application):
         if len(self.get_windows()) > 1:
             state = self.__get_state(window)
             self.__state_cache.append(state)
-            GLib.timeout_add(25000, self.__clean_state_cache, state["id"])
+            GLib.timeout_add(10000, self.__clean_state_cache, state["id"])
             window.destroy()
         else:
-            window.hide()
             self.quit(True)
 
     def __try_closing(self, window, views):
@@ -732,7 +733,7 @@ class Application(Gtk.Application):
                     self.__close_window(window)
         except Exception as e:
             self.__close_window(window)
-            print("Application::__on_forms_filled():", e)
+            Logger.error("Application::__on_forms_filled(): %s", e)
 
     def __on_get_plugins(self, source, result, data):
         """
@@ -743,9 +744,10 @@ class Application(Gtk.Application):
         """
         plugins = source.get_plugins_finish(result)
         for plugin in plugins:
-            print(plugin.get_name(),
-                  plugin.get_description(),
-                  plugin.get_path())
+            Logger.info("%s, %s, %s",
+                        plugin.get_name(),
+                        plugin.get_description(),
+                        plugin.get_path())
 
     def __on_delete_event(self, window, event):
         """
@@ -808,7 +810,7 @@ class Application(Gtk.Application):
 
     def __on_shortcuts_activate(self, action, param):
         """
-            Show help in yelp
+            Show shortcuts
             @param action as Gio.SimpleAction
             @param param as GLib.Variant
         """
@@ -820,17 +822,6 @@ class Application(Gtk.Application):
             shortcuts.set_transient_for(window)
         shortcuts.show()
 
-    def __on_help_activate(self, action, param):
-        """
-            Show help in yelp
-            @param action as Gio.SimpleAction
-            @param param as GLib.Variant
-        """
-        try:
-            Gtk.show_uri(None, "help:eolie", Gtk.get_current_event_time())
-        except:
-            print(_("Eolie: You need to install yelp."))
-
     def __on_about_activate_response(self, dialog, response_id):
         """
             Destroy about dialog when closed
@@ -854,7 +845,9 @@ class Application(Gtk.Application):
             @param application as Gio.Application
         """
         if self.get_windows():
-            self.active_window.present_with_time(Gtk.get_current_event_time())
+            # https://bugzilla.gnome.org/show_bug.cgi?id=766284
+            monotonic_time = int(GLib.get_monotonic_time() / 1000)
+            self.active_window.present_with_time(monotonic_time)
 
     def __on_shortcut_action(self, action, param):
         """
