@@ -1,4 +1,4 @@
-# Copyright (c) 2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2017-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -18,7 +18,7 @@ from eolie.view import View
 from eolie.popover_webview import WebViewPopover
 from eolie.pages_manager import PagesManager
 from eolie.sites_manager import SitesManager
-from eolie.define import El, LoadingType
+from eolie.define import App, LoadingType
 
 
 class DelayedStack(Gtk.Stack):
@@ -72,8 +72,6 @@ class Container(Gtk.Overlay):
         Main Eolie view
     """
 
-    __PRELOADED = 2
-
     def __init__(self, window):
         """
             Ini.container
@@ -85,13 +83,8 @@ class Container(Gtk.Overlay):
         self.__current = None
         self.__next_timeout_id = None
         self.__previous_timeout_id = None
-        self.__preload_timeout_id = None
-        self.__preloaded = []
         self.__pending_items = []
 
-        self.__preloaded_max = El().settings.get_value(
-                                              "preloaded-webviews").get_int32()
-
         self.__stack = DelayedStack()
         self.__stack.set_hexpand(True)
         self.__stack.set_vexpand(True)
@@ -103,20 +96,20 @@ class Container(Gtk.Overlay):
         self.__expose_stack.set_hexpand(True)
         self.__expose_stack.set_vexpand(True)
         self.__expose_stack.set_transition_type(
-                                       Gtk.StackTransitionType.CROSSFADE)
+            Gtk.StackTransitionType.CROSSFADE)
         self.__expose_stack.set_transition_duration(150)
         self.__expose_stack.show()
         self.__pages_manager = PagesManager(self.__window)
         self.__pages_manager.show()
         self.__sites_manager = SitesManager(self.__window)
-        if El().settings.get_value("show-sidebar"):
+        if App().settings.get_value("show-sidebar"):
             self.__sites_manager.show()
-        El().settings.connect("changed::show-sidebar",
-                              self.__on_show_sidebar_changed)
+        App().settings.connect("changed::show-sidebar",
+                               self.__on_show_sidebar_changed)
         paned = Gtk.Paned.new(Gtk.Orientation.HORIZONTAL)
         paned.pack1(self.__sites_manager, False, False)
         paned.add2(self.__expose_stack)
-        position = El().settings.get_value("sidebar-position").get_int32()
+        position = App().settings.get_value("sidebar-position").get_int32()
         paned.set_position(position)
         paned.connect("notify::position", self.__on_paned_notify_position)
         paned.show()
@@ -135,22 +128,17 @@ class Container(Gtk.Overlay):
             @param atime as int
             @return WebView
         """
-        webview = self.__get_webview(ephemeral)
+        webview = View.get_new_webview(ephemeral, self.__window)
         if atime is not None:
             webview.set_atime(atime)
         if state is not None:
             webview.restore_session_state(state)
         if uri is not None:
+            webview.set_uri(uri)
             # Force loading
-            if loading_type == LoadingType.BACKGROUND:
+            if loading_type == LoadingType.BACKGROUND or self.in_expose:
                 webview.load_uri(uri)
-            elif loading_type in [LoadingType.OFFLOAD, LoadingType.FOREGROUND]:
-                webview.set_uri(uri)
         self.add_webview_with_new_view(webview, loading_type)
-        if self.__preload_timeout_id is not None:
-            GLib.source_remove(self.__preload_timeout_id)
-        self.__preload_timeout_id = GLib.timeout_add(2000,
-                                                     self.__preload_webviews)
         return webview
 
     def add_webviews(self, items):
@@ -203,7 +191,7 @@ class Container(Gtk.Overlay):
         # Reason: we do not remove/destroy view to let stack animation run
         count = len(self.__pages_manager.children)
         self.__window.toolbar.actions.count_label.set_text(str(count))
-        El().update_unity_badge()
+        App().update_unity_badge()
 
     def add_view(self, view):
         """
@@ -217,7 +205,7 @@ class Container(Gtk.Overlay):
         self.__stack.set_visible_child(view)
         count = len(self.__stack.get_children())
         self.__window.toolbar.actions.count_label.set_text(str(count))
-        El().update_unity_badge()
+        App().update_unity_badge()
         self.__pages_manager.update_visible_child()
         self.__sites_manager.update_visible_child()
 
@@ -234,11 +222,11 @@ class Container(Gtk.Overlay):
             self.__current = self.__stack.get_visible_child()
             count = len(children)
             self.__window.toolbar.actions.count_label.set_text(str(count))
-            El().update_unity_badge()
+            App().update_unity_badge()
             self.__pages_manager.update_visible_child()
             self.__sites_manager.update_visible_child()
         else:
-            for window in El().windows:
+            for window in App().windows:
                 window.mark(False)
             self.__window.close()
 
@@ -296,8 +284,8 @@ class Container(Gtk.Overlay):
             @param view as View
         """
         page_id = view.webview.get_page_id()
-        El().helper.call("FormsFilled", page_id, None,
-                         self.__on_forms_filled, view)
+        App().helper.call("FormsFilled", page_id, None,
+                          self.__on_forms_filled, view)
 
     def close_view(self, view):
         """
@@ -307,19 +295,19 @@ class Container(Gtk.Overlay):
         """
         # Get children less view
         views = self.__get_children()
-        if view.destroying:
+        if view.destroying or view not in views:
             return
         views.remove(view)
         views_count = len(views)
-        El().history.set_page_state(view.webview.uri)
+        App().history.set_page_state(view.webview.uri)
         self.__window.close_popovers()
         # Needed to unfocus titlebar
         self.__window.set_focus(None)
         was_current = view == self.__window.container.current
         if not view.webview.ephemeral:
-            El().pages_menu.add_action(view.webview.title,
-                                       view.webview.uri,
-                                       view.webview.get_session_state())
+            App().pages_menu.add_action(view.webview.title,
+                                        view.webview.uri,
+                                        view.webview.get_session_state())
 
         # Remove webview from parent
         if view.webview.parent is not None:
@@ -327,9 +315,9 @@ class Container(Gtk.Overlay):
         view.destroy()
         # Don't show 0 as we are going to open a new one
         if views_count:
-            El().update_unity_badge()
+            App().update_unity_badge()
             self.__window.toolbar.actions.count_label.set_text(
-                                                       str(views_count))
+                str(views_count))
         # Nothing to do if was not current page
         if not was_current:
             return False
@@ -366,15 +354,7 @@ class Container(Gtk.Overlay):
             self.__window.container.set_current(next_view, True)
         else:
             # We are last row, add a new one
-            self.add_webview(El().start_page, LoadingType.FOREGROUND)
-
-    def stop_preloading(self):
-        """
-            Stop webview preloading
-        """
-        if self.__preload_timeout_id is not None:
-            GLib.source_remove(self.__preload_timeout_id)
-            self.__preload_timeout_id = None
+            self.add_webview(App().start_page, LoadingType.FOREGROUND)
 
     def next(self):
         """
@@ -383,9 +363,9 @@ class Container(Gtk.Overlay):
         if self.__next_timeout_id is None and\
                 self.__next_timeout_id != -1:
             self.__next_timeout_id = GLib.timeout_add(
-                                                   100,
-                                                   self.__on_prev_next_timeout,
-                                                   self.__pages_manager.next)
+                100,
+                self.__on_prev_next_timeout,
+                self.__pages_manager.next)
         else:
             self.__pages_manager.next()
 
@@ -396,9 +376,9 @@ class Container(Gtk.Overlay):
         if self.__previous_timeout_id is None and\
                 self.__previous_timeout_id != -1:
             self.__previous_timeout_id = GLib.timeout_add(
-                                                 100,
-                                                 self.__on_prev_next_timeout,
-                                                 self.__pages_manager.previous)
+                100,
+                self.__on_prev_next_timeout,
+                self.__pages_manager.previous)
         else:
             self.__pages_manager.previous()
 
@@ -488,31 +468,6 @@ class Container(Gtk.Overlay):
             self.__expose_stack.set_visible_child_name("stack")
             self.__pages_manager.update_visible_child()
 
-    def __get_webview(self, ephemeral):
-        """
-            Get a new webview
-            @param ephemeral as bool
-            @return WebView
-        """
-        if self.__preloaded and not ephemeral:
-            webview = self.__preloaded.pop(0)
-        else:
-            webview = View.get_new_webview(ephemeral, self.__window)
-        return webview
-
-    def __preload_webviews(self):
-        """
-            Preload some webviews
-        """
-        self.__preload_timeout_id = None
-        # Ignore stupid values
-        if self.__preloaded_max < 1 or self.__preloaded_max > 20:
-            return
-        webview = View.get_new_webview(False, self.__window)
-        self.__preloaded.append(webview)
-        if len(self.__preloaded) < self.__preloaded_max:
-            return True
-
     def __get_new_view(self, webview):
         """
             Get a new view
@@ -550,12 +505,12 @@ class Container(Gtk.Overlay):
             @param ignore as GParamInt
         """
         position = paned.get_position()
-        saved_position = El().settings.get_value(
-                                                "sidebar-position").get_int32()
+        saved_position = App().settings.get_value(
+            "sidebar-position").get_int32()
         # We do not want to keep minimal mode changes
         if position >= 80 or saved_position > position:
-            El().settings.set_value("sidebar-position",
-                                    GLib.Variant("i", position))
+            App().settings.set_value("sidebar-position",
+                                     GLib.Variant("i", position))
         self.__sites_manager.set_minimal(position < 80)
 
     def __on_show_sidebar_changed(self, settings, value):
@@ -564,7 +519,7 @@ class Container(Gtk.Overlay):
             @param settings as Gio.Settings
             @param value as bool
         """
-        if El().settings.get_value("show-sidebar"):
+        if App().settings.get_value("show-sidebar"):
             self.__sites_manager.show()
         else:
             self.__sites_manager.hide()
