@@ -1,4 +1,4 @@
-# Copyright (c) 2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2017-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # Copyright (c) 2016 Felipe Borges <felipeborges@gnome.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,9 +19,11 @@ from uuid import uuid4
 from eolie.define import PROXY_BUS, PROXY_PATH, Type
 from eolie.list import LinkedList
 from eolie.helper_passwords import PasswordsHelper
+from eolie.logger import Logger
 
 
 class Server:
+
     def __init__(self, con, path):
         method_outargs = {}
         method_inargs = {}
@@ -31,7 +33,7 @@ class Server:
                 method_outargs[method.name] = '(' + ''.join(
                               [arg.signature for arg in method.out_args]) + ')'
                 method_inargs[method.name] = tuple(
-                                       arg.signature for arg in method.in_args)
+                    arg.signature for arg in method.in_args)
 
             con.register_object(object_path=path,
                                 interface_info=interface,
@@ -89,6 +91,9 @@ class ProxyExtensionServer(Server):
     </method>
     <method name="SetNextElementValue">
     </method>
+    <method name="EnableJS">
+        <arg type="s" name="netloc" direction="in" />
+    </method>
     <method name="SaveCredentials">
       <arg type="s" name="uuid" direction="in" />
       <arg type="s" name="user_form_name" direction="in" />
@@ -145,7 +150,8 @@ class ProxyExtensionServer(Server):
         self.__page = page
         self.__form_extension = form_extension
         self.__jsblock_extension = jsblock_extension
-        self.__focused = None
+        self.__focus_element = None
+        self.__mouse_down_element = None
         self.__on_input_timeout_id = None
         self.__elements_history = {}
         self.__send_requests = []
@@ -154,12 +160,12 @@ class ProxyExtensionServer(Server):
         addr = Gio.dbus_address_get_for_bus_sync(Gio.BusType.SESSION, None)
         self.__bus = None
         Gio.DBusConnection.new_for_address(
-                               addr,
-                               Gio.DBusConnectionFlags.AUTHENTICATION_CLIENT |
-                               Gio.DBusConnectionFlags.MESSAGE_BUS_CONNECTION,
-                               None,
-                               None,
-                               self.__on_bus_new_for_address)
+            addr,
+            Gio.DBusConnectionFlags.AUTHENTICATION_CLIENT |
+            Gio.DBusConnectionFlags.MESSAGE_BUS_CONNECTION,
+            None,
+            None,
+            self.__on_bus_new_for_address)
         form_extension.connect("submit-form", self.__on_submit_form)
         page.connect("send-request", self.__on_send_request)
         page.connect("context-menu", self.__on_context_menu)
@@ -179,36 +185,45 @@ class ProxyExtensionServer(Server):
                 return True
         return False
 
+    def EnableJS(self, netloc):
+        """
+            Enable JS for netloc
+            @param netloc as JS
+        """
+        self.__jsblock_extension.enable_for(netloc)
+
     def SaveCredentials(self, uuid, user_form_name,
-                        pass_form_name, uri, form_uri):
+                        pass_form_name, hostname_uri, form_uri):
         """
             Save credentials to org.freedesktop.Secrets
             @param uuid as str
             @param user_form_name as str
             @param pass_form_name as str
-            @param uri as str
+            @param hostname_uri as str
             @param form_uri as str
         """
         if self.__form_extension.pending_credentials in [None, Type.NONE]:
             return
         try:
-            (_uuid, _user_form_name, user_form_value,
-             _pass_form_name, pass_form_value,
-             _uri, _form_uri) = self.__form_extension.pending_credentials
+            (_uuid,
+             _user_form_name,
+             user_form_value,
+             _pass_form_name,
+             pass_form_value,
+             _hostname_uri,
+             _form_uri) = self.__form_extension.pending_credentials
             if user_form_name != _user_form_name or\
                     pass_form_name != _pass_form_name or\
-                    uri != _uri or\
+                    hostname_uri != _hostname_uri or\
                     form_uri != _form_uri:
                 return
-            parsed = urlparse(uri)
-            uri = "%s://%s" % (parsed.scheme, parsed.netloc)
             if not uuid:
                 uuid = str(uuid4())
                 self.__helper.store(user_form_name,
                                     user_form_value,
                                     pass_form_name,
                                     pass_form_value,
-                                    uri,
+                                    hostname_uri,
                                     form_uri,
                                     uuid,
                                     None)
@@ -219,12 +234,12 @@ class ProxyExtensionServer(Server):
                                     user_form_value,
                                     pass_form_name,
                                     pass_form_value,
-                                    uri,
+                                    hostname_uri,
                                     form_uri,
                                     uuid,
                                     None)
         except Exception as e:
-            print("ProxyExtension::SaveCredentials():", e)
+            Logger.error("ProxyExtension::SaveCredentials(): %s", e)
 
     def SetAuthForms(self, userform, username):
         """
@@ -239,7 +254,7 @@ class ProxyExtensionServer(Server):
             (forms, textareas) = self.__form_extension.get_elements(elements)
             for form in forms:
                 if form["username"].get_name() == userform:
-                    self.__helper.get(form["element"].get_action(),
+                    self.__helper.get(self.__form_extension.get_form_uri(form),
                                       userform,
                                       form["password"].get_name(),
                                       self.__form_extension.set_input_forms,
@@ -248,7 +263,7 @@ class ProxyExtensionServer(Server):
                                       username)
                     return
         except Exception as e:
-            print("ProxyExtension::SetAuthForms():", e)
+            Logger.error("ProxyExtension::SetAuthForms(): %s", e)
 
     def GetScripts(self):
         """
@@ -258,7 +273,7 @@ class ProxyExtensionServer(Server):
         try:
             return self.__jsblock_extension.scripts
         except Exception as e:
-            print("ProxyExtension::GetScripts():", e)
+            Logger.error("ProxyExtension::GetScripts(): %s", e)
         return []
 
     def GetImages(self):
@@ -278,7 +293,7 @@ class ProxyExtensionServer(Server):
                     uris.append(uri)
             return uris
         except Exception as e:
-            print("ProxyExtension::GetImages():", e)
+            Logger.error("ProxyExtension::GetImages(): %s", e)
         return []
 
     def GetVideos(self):
@@ -328,7 +343,7 @@ class ProxyExtensionServer(Server):
                     uris.append(uri)
             return uris
         except Exception as e:
-            print("ProxyExtension::GetImagesLinks():", e)
+            Logger.error("ProxyExtension::GetImagesLinks(): %s", e)
         return []
 
     def GetSelection(self):
@@ -362,38 +377,40 @@ class ProxyExtensionServer(Server):
         """
             Set focused form to previous value
         """
-        if self.__focused is None:
+        if self.__focus_element is None:
             return
         try:
-            value = self.__focused.get_value().rstrip(" ")
-            if self.__focused in self.__elements_history.keys():
-                current = self.__elements_history[self.__focused]
+            value = self.__focus_element.get_value().rstrip(" ")
+            if self.__focus_element in self.__elements_history.keys():
+                current = self.__elements_history[self.__focus_element]
                 if current.prev is not None:
-                    self.__elements_history[self.__focused] = current.prev
-                    self.__focused.set_value(current.prev.value)
+                    self.__elements_history[self.__focus_element] =\
+                        current.prev
+                    self.__focus_element.set_value(current.prev.value)
             elif value:
                 next = LinkedList(value, None, None)
                 current = LinkedList("", next, None)
                 next.set_prev(current)
-                self.__elements_history[self.__focused] = current
-                self.__focused.set_value("")
+                self.__elements_history[self.__focus_element] = current
+                self.__focus_element.set_value("")
         except Exception as e:
-            print("ProxyExtension::SetPreviousForm():", e)
+            Logger.error("ProxyExtension::SetPreviousForm(): %s", e)
 
     def SetNextElementValue(self):
         """
             Set focused form to next value
         """
-        if self.__focused is None:
+        if self.__focus_element is None:
             return
         try:
-            if self.__focused in self.__elements_history.keys():
-                current = self.__elements_history[self.__focused]
+            if self.__focus_element in self.__elements_history.keys():
+                current = self.__elements_history[self.__focus_element]
                 if current.next is not None:
-                    self.__elements_history[self.__focused] = current.next
-                    self.__focused.set_value(current.next.value)
+                    self.__elements_history[self.__focus_element] =\
+                        current.next
+                    self.__focus_element.set_value(current.next.value)
         except Exception as e:
-            print("ProxyExtension::SetNextForm():", e)
+            Logger.error("ProxyExtension::SetNextForm(): %s", e)
 
 #######################
 # PRIVATE             #
@@ -405,7 +422,8 @@ class ProxyExtensionServer(Server):
             @param textareas as [WebKit2WebExtension.DOMHTMLTextAreaElement]
             @param webpage as WebKit2WebExtension.WebPage
         """
-        self.__focused = None
+        self.__mouse_down_element = None
+        self.__focus_element = None
         self.__elements_history = {}
 
         parsed = urlparse(webpage.get_uri())
@@ -450,29 +468,32 @@ class ProxyExtensionServer(Server):
 
     def __on_focus(self, element, event):
         """
-            Keep last focused form
+            Warn user about a password form on an http page and
+            update focused element
             @param element as WebKit2WebExtension.DOMElement
             @param event as WebKit2WebExtension.DOMUIEvent
         """
+        self.__focus_element = element
         if isinstance(element, WebKit2WebExtension.DOMHTMLInputElement):
             if element.get_input_type() == "password" and\
                     self.__bus is not None:
                 self.__bus.emit_signal(
-                              None,
-                              PROXY_PATH,
-                              self.__proxy_bus,
-                              "UnsecureFormFocused",
-                              None)
-        self.__focused = element
+                    None,
+                    PROXY_PATH,
+                    self.__proxy_bus,
+                    "UnsecureFormFocused",
+                    None)
 
     def __on_blur(self, element, event):
         """
-            Reset focus
+            Loose focus so reset mouse down element
             @param element as WebKit2WebExtension.DOMElement
             @param event as WebKit2WebExtension.DOMUIEvent
         """
-        if self.__focused == element:
-            self.__focused = None
+        if self.__mouse_down_element == element:
+            self.__mouse_down_element = None
+        if self.__focus_element == element:
+            self.__focus_element = None
 
     def __on_mouse_down(self, element, event):
         """
@@ -482,7 +503,7 @@ class ProxyExtensionServer(Server):
         """
         if element.get_name() is None:
             return
-        if self.__focused != element and\
+        if self.__mouse_down_element != element and\
                 self.__bus is not None:
             form_uri = element.get_form().get_action()
             parsed_form_uri = urlparse(form_uri)
@@ -490,11 +511,12 @@ class ProxyExtensionServer(Server):
                                     parsed_form_uri.netloc)
             args = GLib.Variant("(ss)", (element.get_name(), form_uri))
             self.__bus.emit_signal(
-                              None,
-                              PROXY_PATH,
-                              self.__proxy_bus,
-                              "ShowCredentials",
-                              args)
+                None,
+                PROXY_PATH,
+                self.__proxy_bus,
+                "ShowCredentials",
+                args)
+        self.__mouse_down_element = element
 
     def __on_input(self, element, event):
         """
@@ -502,6 +524,8 @@ class ProxyExtensionServer(Server):
             @param element as WebKit2WebExtension.DOMElement
             @param event as WebKit2WebExtension.DOMUIEvent
         """
+        # input signal does not always work
+        self.__focus_element = element
         if self.__on_input_timeout_id is not None:
             GLib.source_remove(self.__on_input_timeout_id)
         self.__on_input_timeout_id = GLib.timeout_add(500,
@@ -515,6 +539,8 @@ class ProxyExtensionServer(Server):
         """
         self.__on_input_timeout_id = None
         new_value = element.get_value()
+        if new_value is None:
+            return
         item = LinkedList("", None, None)
         if element in self.__elements_history.keys():
             item = self.__elements_history[element]
@@ -533,9 +559,9 @@ class ProxyExtensionServer(Server):
         self.__add_event_listeners(forms, textareas, webpage)
         for form in forms:
             form["element"].add_event_listener(
-                                          "submit",
-                                          self.__form_extension.on_form_submit,
-                                          False)
+                "submit",
+                self.__form_extension.on_form_submit,
+                False)
             self.__form_extension.set_credentials(form, webpage)
 
     def __on_context_menu(self, webpage, context_menu, hit):
@@ -556,11 +582,11 @@ class ProxyExtensionServer(Server):
         """
         if self.__bus is not None:
             self.__bus.emit_signal(
-                                  None,
-                                  PROXY_PATH,
-                                  self.__proxy_bus,
-                                  "AskSaveCredentials",
-                                  variant)
+                None,
+                PROXY_PATH,
+                self.__proxy_bus,
+                "AskSaveCredentials",
+                variant)
 
     def __on_notify_uri(self, webpage, param):
         """
